<?php

use EventPortal\Domain\Approval\Handlers\SetEventApprovalStatusToAwaiting;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Input\Text;
use EventPortal\Domain\Component\Input\TextArea;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Handlers\UpdateModel;
use EventPortal\Enums\InertiaMethod;

Form::make('event-approval')
    ->addHandler(new UpdateModel)
    ->addHandler(new SetEventApprovalStatusToAwaiting)
    ->addFields(function (Event $event) {
        return [
            Section::make('General details')->fields([
                Text::make('Meeting Owner initials')
                    ->placeholder('EABL')
                    ->disabled(true)
                    ->value($event->meetingLead->admin->username ?? ''),
                Text::make('Name of third party agency assisting Novo Nordisk with the meeting:')
                    ->placeholder('Penguins')
                    ->name('third_party_names')
                    ->rules(['required']),
                Text::make('Title of meeting:')
                    ->name('name')
                    ->placeholder('Congress')
                    ->disabled(true),
                Text::make('Date of meeting')
                    ->value($event->readable_dates)
                    ->disabled(true)
                    ->placeholder('Congress'),
            ]),
            Section::make('Venue Details')->fields([
                TextArea::make('Venue name and address')
                    ->placeholder('A snappy one-liner to draw attention')
                    ->value($event->venues->implode('nameAndAddress', ' ' . PHP_EOL) ?? '')
                    ->disabled(true),
                TextArea::make('Justification for selecting venue')
                    ->placeholder('A longer event description')
                    ->name('venue_justification')
                    ->rules(['required']),
                Text::make('Room hire costs')
                    ->placeholder('£1234')
                    ->name('room_hire_costs')
                    ->rules(['required']),
                Text::make('Day delegate rate costs')
                    ->placeholder('£1234')
                    ->name('day_delegate_costs')
                    ->rules(['required']),
                TextArea::make('Accommodation details')
                    ->placeholder('Please enter details')
                    ->name('accommodation_details')
                    ->rules(['required']),
                TextArea::make('Subsistence')
                    ->placeholder('Please enter subsistence')
                    ->name('subsistence')
                    ->rules(['required']),
            ]),
            Section::make('Deletage details')->fields([
                TextArea::make('Delegate details')
                    ->placeholder('Please enter details')
                    ->disabled(true),
                TextArea::make('External consultant details')
                    ->placeholder('Please enter details')
                    ->name('extra_consultant_details'),
                TextArea::make(
                    'Any other supporting information (e.g. are delegates from other countries attending this event)'
                )
                    ->name('delegate_supporting_information')
                    ->placeholder('Please enter details'),
            ]),
            Section::make('Approval Details')->fields([
                Text::make('Phone')
                    ->placeholder('01234 567 890'),
                Text::make('Email')
                    ->placeholder('email@example.com')
                    ->rules(['required', 'email']),
            ]),
        ];
    })
    ->addActions(function (Event $event) {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.event.show', $event->id),
        ];
    })
    ->addPrimaryAction(function (Event $event) {
        return Button::make('Submit')
            ->method(InertiaMethod::POST())
            ->route('admin.event.approval.store', $event->id);
    });
