<?php

namespace EventPortal\Domain\Approval\Models;

use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Comment\Models\Comment;
use EventPortal\Domain\Event\Models\Event;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Database\Eloquent\Relations\Pivot;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Approval extends Pivot implements AuditableInterface, Statusable
{
    use Statable;
    use HasStatus;
    use Auditable;
    use Statable;
    use AddToFillable;

    const KEY = 'eventApproval';

    /** @var string */
    protected $table = 'approvals';

    /** @var array<string> */
    protected $fillable = [
        'event_id',
        'admin_id',
        'status',
    ];

    public $incrementing = true;

    public function comments(): MorphMany
    {
        return $this->morphMany(Comment::class, 'commentable');
    }

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function data(): HasMany
    {
        return $this->hasMany(ApprovalData::class, 'approval_id');
    }

    public function transitionRouteName(): string
    {
        return 'admin.event-approval.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    protected function getGraph(): string
    {
        return 'eventApproval';
    }
}
