<?php

namespace EventPortal\Http\Controllers\Admin\EventVenue;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventVenue;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Inertia\Response;

class EventVenueController extends Controller
{
    public function index(Event $event): Response
    {
        $table = Table::make('event-venue.index')
            ->context($event)
            ->query($event->venues()->getQuery())
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Venues')
            ->get();

        $addAVenueButton = Button::make('Add venue to event')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.event.venue.create', $event);

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/EventVenue/Index')
            ->key('event.venue.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->attributes(['event' => $event])
            ->withQuickButton($addAVenueButton)
            ->withSideBar($sidebar)
            ->get();
    }

    public function show(Event $event, EventVenue $eventVenue): Response
    {
        $venueData = Form::make('event-venue.show')
            ->model($eventVenue)
            ->context($event, $eventVenue)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Venues', route('admin.event.venue.index', $event))
            ->add($eventVenue->name->name)
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/EventVenue/Show')
            ->key('event.venue.show')
            ->withData($venueData)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($eventVenue->venue->name)
            ->withSideBar($sidebar)
            ->get();
    }
}
