<?php

namespace EventPortal\Tests\Feature\Admin\Venue;

use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class VenueUpdateTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test */
    public function a_venue_can_be_updated(): void
    {
        $venue = factory(Venue::class)->state('has-address')->create([
            'name' => 'Original Name',
            'website' => 'http://original.example.com',
        ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.venue.update', $venue), [
                'name' => 'New Name',
                'website' => 'http://new.example.com',
                'line_1' => 'Line one',
                'city' => 'London',
                'country' => 'England',
                'area_code' => '1234',
            ])
            ->assertOk();

        $this->assertDatabaseHas('venues', [
            'name' => 'New Name',
            'website' => 'http://new.example.com',
        ]);
    }

    /** @test **/
    public function the_venue_name_is_required(): void
    {
        $venue = factory(Venue::class)->create([
            'name' => 'Original Name',
        ]);

        $this
            ->actingAsSuperAdmin()
            ->post(route('admin.venue.update', $venue), [
                'name' => null,
            ])
            ->assertSessionHasErrors([
                'name',
            ]);

        $this->assertDatabaseHas('venues', [
            'name' => 'Original Name',
        ]);
    }
}
