<?php

namespace EventPortal\Tests;

use DMS\PHPUnitExtensions\ArraySubset\ArraySubsetAsserts;
use EventPortal\Concerns\LoadConfigs;
use EventPortal\Domain\User\Models\User;
use EventPortal\Providers\EventPortalServiceProvider;
use EventPortal\Tests\Support\Assertions\CollectionAssertions;
use EventPortal\Tests\Support\Assertions\PropAssertions;
use EventPortal\Tests\Support\Traits\InteractsWithAdminAuthentication;
use EventPortal\Tests\Support\Traits\InteractsWithAgendas;
use EventPortal\Tests\Support\Traits\InteractsWithDelegateAuthentication;
use EventPortal\Tests\Support\Traits\InteractsWithEventDelegates;
use EventPortal\Tests\Support\Traits\InteractsWithEvents;
use EventPortal\Tests\Support\Traits\InteractsWithEventSpeakers;
use EventPortal\Tests\Support\Traits\InteractsWithEventVenues;
use EventPortal\Tests\Support\Traits\InteractsWithModels;
use EventPortal\Tests\Support\Traits\InteractsWithPathways;
use EventPortal\Tests\Support\Traits\InteractsWithSessions;
use Illuminate\Support\Collection;
use Illuminate\Testing\TestResponse;
use ReflectionClass;
use ReflectionMethod;

class TestCase extends \Orchestra\Testbench\TestCase
{
    use ArraySubsetAsserts,
        InteractsWithModels,
        InteractsWithEvents,
        InteractsWithPathways,
        InteractsWithSessions,
        InteractsWithEventDelegates,
        InteractsWithEventSpeakers,
        InteractsWithAdminAuthentication,
        InteractsWithDelegateAuthentication,
        InteractsWithEventVenues,
        LoadConfigs;

    public function setUp(): void
    {
        parent::setUp();

        $this->setupDatabase();

        $this->app->booted(function (): void {
            $this->loadForms();
            $this->loadQuickButtons();
            $this->loadTables();
            $this->loadSidebars();
            $this->loadStatistics();
            $this->loadNotifications();
            $this->loadMenus();
        });

        TestResponse::mixin(new PropAssertions);
        Collection::mixin(new CollectionAssertions);
    }

    protected function getEnvironmentSetUp($app): void
    {
        $permissions = require './config/permissions.php';
        $roles = require './config/roles.php';
        $permission = require './config/permission.php';
        $stateMachine = require './config/state-machine.php';
        $adminHeaders = require './config/page-headers.php';
        $emails = require './config/emails.php';
        $errors = require './config/errors.php';
        $declineForm = require './config/forms/decline.php';
        $registrationForm = require './config/forms/registration.php';

        $app->config->set('permissions', $permissions);
        $app->config->set('roles', $roles);
        $app->config->set('permission', $permission);
        $app->config->set('state-machine', $stateMachine);
        $app->config->set('admin-headers', $adminHeaders);
        $app->config->set('emails', $emails);
        $app->config->set('errors', $errors);
        $app->config->set('forms.decline', $declineForm);
        $app->config->set('forms.registration', $registrationForm);
        $app->config->set('auth.providers.users.model', User::class);
    }

    protected function getPackageProviders($app): array
    {
        return [EventPortalServiceProvider::class];
    }

    private function setupDatabase(): void
    {
        $this->loadMigrationsFrom(base_path('/database/migrations'));

        include_once 'vendor/iben12/laravel-statable/database/migrations/create_state_history_table.php.stub';
        (new \CreateStateHistoryTable())->up();

        include_once './tests/Migrations/2020_07_23_084500_create_test_model_table.php';
        (new \CreateTestModelTable())->up();
    }

    protected function getPrivateMethod($className, $methodName): ReflectionMethod
    {
        $reflector = new ReflectionClass($className);
        $method = $reflector->getMethod($methodName);
        $method->setAccessible(true);

        return $method;
    }

    public function from(string $url): self
    {
        session()->setPreviousUrl(url($url));

        return $this;
    }

    public function fromRoute(string $route): self
    {
        return $this->from(route($route));
    }
}
