<?php

namespace EventPortal\Tests\Unit\Approval\Handlers;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Approval\Handlers\SetEventApprovalStatusToAwaiting;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class SetEventApprovalToAwaitingTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function sets_as_awaiting(): void
    {
        $event = factory(Event::class)->create();

        $this->assertDatabaseMissing(
            'approvals',
            [
                'id' => $event->id,
            ]
        );

        $payload = Payload::make()->withModel($event);

        app(Pipeline::class)
            ->send($payload)
            ->through([new SetEventApprovalStatusToAwaiting])
            ->thenReturn();

        $this->assertEquals('awaiting-approval', $event->approval->status);
    }

    /** @test **/
    public function only_one_approval_is_create_if_retried(): void
    {
        $event = factory(Event::class)->create();

        $this->assertDatabaseMissing(
            'approvals',
            [
                'event_id' => $event->id,
            ]
        );

        $payload = Payload::make()->withModel($event);

        app(Pipeline::class)
            ->send($payload)
            ->through([new SetEventApprovalStatusToAwaiting])
            ->thenReturn();

        app(Pipeline::class)
            ->send($payload)
            ->through([new SetEventApprovalStatusToAwaiting])
            ->thenReturn();

        $countInDB = $this->assertDatabaseHas(
            'approvals',
            [
                'event_id' => $event->id,
                'status' => 'awaiting-approval',
            ]
        )->count();

        $this->assertSame(1, $countInDB);

        $this->assertEquals('awaiting-approval', $event->approval->status);
    }
}
